print("import: 'compliance_checker'")
import compliance_checker

print("import: 'compliance_checker.cf'")
import compliance_checker.cf

print("import: 'compliance_checker.tests'")
import compliance_checker.tests

